package com.huaxia;

import com.huaxia.imes.mqtt.MqttConsumerCallBack;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;

/**
 * @author zx
 * @since 2024年11月13日
 * MQTT 消费端配置
 */
@Configuration
@Slf4j
public class MqttConsumerConfig {

    @Value("${mqtt.url}")
    private String url;
    @Value("${mqtt.username}")
    private String username;
    @Value("${mqtt.password}")
    private String password;
    @Value("${mqtt.client.id}")
    private String clientId;
    @Value("${mqtt.client.topic.default}")
    private String topic;

    /**
     * mqtt 客户端
     */
    private MqttClient mqttClient;


    /**
     * 在bean初始化后连接mqtt服务器
     */
    @PostConstruct
    public void init(){
        connect();
    }


    /**
     * 连接mqtt服务器
     */
    private void connect() {
        try {
            // 创建客户端
            mqttClient=new MqttClient(url,clientId,new MemoryPersistence());
            //连接参数设置
            MqttConnectOptions options=new MqttConnectOptions();
            // 设置是否清空session，这里如果设置为false表示服务器会保留客户端的连接记录，这里设置为true表示每次连接到服务器都以新的身份连接
            options.setCleanSession(false);
            //设置连接用户名和密码
            options.setUserName(username);
            options.setPassword(password.toCharArray());
            //设置超时时间单位为秒
            options.setConnectionTimeout(30);
            //设置会话心跳时间 单位为秒 服务器会每隔1.5*20秒的时间向客户端发送心跳判断客户端是否在线，但这个方法并没有重连的机制
            options.setKeepAliveInterval(60);
            //设置遗嘱消息的话题，若客户端和服务器之间的连接意外中断，服务器将发布客户端的遗嘱信息
            options.setWill(topic, (clientId+"与服务器断开连接").getBytes(), 0, false);
            //设置回调
            mqttClient.setCallback(new MqttConsumerCallBack());
            // 连接服务器
            mqttClient.connect(options);
            // 订阅主题
            //消息等级
            int[] qos={1,1,2};
            //主题
            final String topics[] = {topic, topic + "/+"/*, TestTopic + "/r1"*/, topic + "/r/+"/*, TestTopic + "/r2"*/};
            //订阅主题
            mqttClient.subscribe(topics,qos);
        }catch (Exception e){
           log.error("连接异常{}",e.getMessage());
        }

    }
}
